import datetime
import json
from unittest import TestCase

import responses
from click.testing import CliRunner

from montecarlodata.secrets.commands import (
    create_secret,
    delete_secret,
    get_secret,
    list_secrets,
)
from montecarlodata.tools import format_date, format_datetime
from tests.test_common_user import _SAMPLE_CONFIG


class SecretsTest(TestCase):
    @responses.activate
    def test_create_secret(self):
        expires_at = datetime.datetime.utcnow().isoformat()
        created_at = datetime.datetime.utcnow().isoformat()
        updated_at = datetime.datetime.utcnow().isoformat()
        responses.post(
            _SAMPLE_CONFIG.mcd_api_endpoint,
            json={
                "data": {
                    "createAccountSecret": {
                        "secret": {
                            "name": "a-secret",
                            "scope": "global",
                            "description": "a description",
                            "expiresAt": expires_at,
                            "createdBy": "test-user_1",
                            "createdAt": created_at,
                            "updatedBy": "test-user_2",
                            "lastUpdate": updated_at,
                        },
                    },
                }
            },
        )

        ctx = _SAMPLE_CONFIG
        runner = CliRunner()
        result = runner.invoke(
            create_secret,
            obj={"config": ctx},
            args=["--name", "a-secret"],
            input="secret_value",
        )

        self.assertEqual(result.output, "Secret value: \nCreated secret: 'a-secret'.\n")
        self.assertEqual(
            json.loads(responses.calls[0].request.body.decode("utf8"))["variables"],
            {
                "description": None,
                "expiresAt": None,
                "name": "a-secret",
                "scope": "global",
                "value": "secret_value",
            },
        )

    @responses.activate
    def test_get_secret_reveal(self):
        expires_at = datetime.datetime.utcnow().isoformat()
        created_at = datetime.datetime.utcnow().isoformat()
        updated_at = datetime.datetime.utcnow().isoformat()
        responses.post(
            _SAMPLE_CONFIG.mcd_api_endpoint,
            json={
                "data": {
                    "getAccountSecret": {
                        "name": "a-secret",
                        "scope": "global",
                        "description": "a description",
                        "expiresAt": expires_at,
                        "createdBy": "test-user_1",
                        "createdAt": created_at,
                        "updatedBy": "test-user_2",
                        "lastUpdate": updated_at,
                        "value": "secret value",
                    },
                }
            },
        )

        ctx = _SAMPLE_CONFIG
        runner = CliRunner()
        result = runner.invoke(
            get_secret,
            obj={"config": ctx},
            args=["--name", "a-secret", "--reveal"],
        )

        self.assertEqual(result.output, "secret value\n")
        self.assertEqual(
            json.loads(responses.calls[0].request.body.decode("utf8"))["variables"],
            {"name": "a-secret", "reveal": True},
        )

        @responses.activate
        def test_get_secret_no_reveal(self):
            expires_at = datetime.datetime.utcnow().isoformat()
            created_at = datetime.datetime.utcnow().isoformat()
            updated_at = datetime.datetime.utcnow().isoformat()
            responses.post(
                _SAMPLE_CONFIG.mcd_api_endpoint,
                json={
                    "data": {
                        "getAccountSecret": {
                            "name": "a-secret",
                            "scope": "global",
                            "description": "a description",
                            "expiresAt": expires_at,
                            "createdBy": "test-user_1",
                            "createdAt": created_at,
                            "updatedBy": "test-user_2",
                            "lastUpdate": updated_at,
                        },
                    }
                },
            )

            ctx = _SAMPLE_CONFIG
            runner = CliRunner()
            result = runner.invoke(
                get_secret,
                obj={"config": ctx},
                args=["--name", "a-secret"],
            )

            self.assertEqual(
                result.output,
                "Name: a-secret\n"
                "Scope: global\n"
                "Description: a description\n"
                f"Expires at: {format_date(expires_at)}\n"
                "Created by: test-user_1\n"
                f"Created at: {format_datetime(created_at)}\n"
                "Last updated by: test-user_2\n"
                f"Last updated at: {format_datetime(created_at)}\n",
            )
            self.assertEqual(
                json.loads(responses.calls[0].request.body.decode("utf8"))["variables"],
                {"name": "a-secret", "reveal": False},
            )

    @responses.activate
    def test_list_secrets(self):
        expires_at = datetime.datetime.utcnow().isoformat()
        created_at = datetime.datetime.utcnow().isoformat()
        updated_at = datetime.datetime.utcnow().isoformat()
        responses.post(
            _SAMPLE_CONFIG.mcd_api_endpoint,
            json={
                "data": {
                    "getAccountSecrets": [
                        {
                            "name": "a-secret",
                            "scope": "global",
                            "description": "a description",
                            "expiresAt": expires_at,
                            "createdBy": "test-user_1",
                            "createdAt": created_at,
                            "updatedBy": "test-user_2",
                            "lastUpdate": updated_at,
                        }
                    ],
                }
            },
        )

        ctx = _SAMPLE_CONFIG
        runner = CliRunner()
        result = runner.invoke(
            list_secrets,
            obj={"config": ctx},
        )

        self.assertEqual(
            result.output,
            "╒══════════╤═════════╤═══════════════╤══════════════╤══════════════╤═════════════════════╤═══════════════════╤═════════════════════╕\n"
            "│ Name     │ Scope   │ Description   │ Expires at   │ Created by   │ Created at          │ Last updated by   │ Last updated at     │\n"
            "╞══════════╪═════════╪═══════════════╪══════════════╪══════════════╪═════════════════════╪═══════════════════╪═════════════════════╡\n"
            f"│ a-secret │ global  │ a description │ {format_date(expires_at)}   │ test-user_1  │ {format_datetime(created_at)} │ test-user_2       │ {format_datetime(created_at)} │\n"
            "╘══════════╧═════════╧═══════════════╧══════════════╧══════════════╧═════════════════════╧═══════════════════╧═════════════════════╛\n",
        )
        self.assertEqual(
            json.loads(responses.calls[0].request.body.decode("utf8"))["variables"],
            None,
        )

    @responses.activate
    def test_delete_secret(self):
        responses.post(
            _SAMPLE_CONFIG.mcd_api_endpoint,
            json={
                "data": {
                    "deleteAccountSecret": {
                        "deleted": True,
                    },
                }
            },
        )

        ctx = _SAMPLE_CONFIG
        runner = CliRunner()
        result = runner.invoke(delete_secret, obj={"config": ctx}, args=["--name", "a-secret"])

        self.assertEqual(result.output, "Deleted secret: 'a-secret'.\n")
        self.assertEqual(
            json.loads(responses.calls[0].request.body.decode("utf8"))["variables"],
            {"name": "a-secret"},
        )

    @responses.activate
    def test_delete_secret_not_found(self):
        responses.post(
            _SAMPLE_CONFIG.mcd_api_endpoint,
            json={
                "data": {
                    "deleteAccountSecret": {
                        "deleted": False,
                    },
                }
            },
        )

        ctx = _SAMPLE_CONFIG
        runner = CliRunner()
        result = runner.invoke(delete_secret, obj={"config": ctx}, args=["--name", "a-secret"])

        self.assertEqual(result.output, "Secret 'a-secret' was not found.\n")
        self.assertEqual(
            json.loads(responses.calls[0].request.body.decode("utf8"))["variables"],
            {"name": "a-secret"},
        )



MUTATION_TRIGGER_CAAS_MIGRATION_TEST = """
    mutation triggerPlatformMigrationTest($dcUuid: UUID) {
        triggerPlatformMigrationTest(
            dcUuid: $dcUuid
        ) {
            migrationUuid
        }
    }
"""

QUERY_CAAS_MIGRATION_TEST_STATUS = """
    query getPlatformMigrationStatus($dcUuid: UUID, $migrationUuid: UUID!) {
      getPlatformMigrationStatus(
        dcUuid: $dcUuid
        migrationUuid: $migrationUuid
      ) {
        output
      }
    }
"""

QUERY_GET_SERVICES = """
query getUser {
  getUser {
    account {
      dataCollectors {
        uuid
        deploymentType
        active
        stackArn
        agents {
          isDeleted
          endpoint
        }
      }
    }
  }
}
"""



import uuid
from unittest import TestCase
from unittest.mock import Mock

from box import Box
from pycarlo.core import Client

from montecarlodata.collector.validation import CollectorValidationService
from montecarlodata.common.data import MonolithResponse
from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
    EXPECTED_ADD_STREAMING_SYSTEM_RESPONSE_FIELD,
)
from montecarlodata.integrations.onboarding.streaming.streamings import (
    StreamingOnboardingService,
)
from montecarlodata.queries.onboarding import (
    ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
    ADD_STREAMING_SYSTEM_MUTATION,
)
from montecarlodata.utils import AwsClientWrapper, GqlWrapper
from tests.test_common_user import _SAMPLE_CONFIG


class StreamingOnboardingTest(TestCase):
    _EXPECTED_NEW_CONNECTION_RESPONSE = {
        "connection": {
            "id": "9511",
            "type": "CONFLUENT_KAFKA_CONNECT",
            "uuid": "de7b4ebb-1574-45c1-9433-07fae8b3fc9f",
            "streamingCluster": {
                "uuid": "c5c9f51d-06d6-47f6-90c1-27fda4160dda",
                "clusterType": "CONFLUENT_KAFKA_CONNECT",
                "externalClusterId": "test_cluster_id",
                "streamingSystemUuid": "6f5d117f-e644-41b4-aa60-f593c0f0dfd1",
            },
        }
    }

    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)
        self._mc_client_mock = Mock(autospec=Client)
        self._validation_service_mock = Mock(autospec=CollectorValidationService)

        self._service = StreamingOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            mc_client=self._mc_client_mock,
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )
        self._service._validation_service = self._validation_service_mock

    def test_create_streaming_system(self):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data=Box(
                {
                    "streamingSystem": {
                        "id": "1",
                        "createdTime": "2023-11-14T23:41:05.465076+00:00",
                        "updatedTime": "2023-11-14T23:41:05.465093+00:00",
                        "name": "test system 2",
                        "type": "CONFLUENT_CLOUD",
                        "accountUuid": "a5cbd8cc-8e91-4a41-aca4-4bf5bd320578",
                        "dcId": "c16e3efc-0d54-4088-9514-372a38506e62",
                        "deletedAt": None,
                        "uuid": "6f5d117f-e644-41b4-aa60-f593c0f0dfd1",
                    }
                }
            ),
            errors=[],
        )

        system_type = "confluent-cloud"
        system_name = "test confluent system"
        dc_id = str(uuid.uuid4())
        self._service.create_streaming_system(system_type, system_name, dc_id)

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=ADD_STREAMING_SYSTEM_MUTATION,
            operation=EXPECTED_ADD_STREAMING_SYSTEM_RESPONSE_FIELD,
            service="streaming_onboarding_service",
            variables=dict(
                streaming_system_type=system_type,
                streaming_system_name=system_name,
                dc_id=dc_id,
            ),
        )

    def test_onboard_streaming_cluster_connection_with_key(self):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data=Box(self._EXPECTED_NEW_CONNECTION_RESPONSE), errors=[]
        )

        self._service.onboard_streaming_cluster_connection(
            connection_type="confluent-kafka-connect",
            key="tmp/mykey",
            dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
            new_streaming_system_name="test",
            new_streaming_system_type="confluent-cloud",
            new_cluster_id="test_cluster_id",
            new_cluster_name="test_cluster_name",
        )

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
            service="streaming_onboarding_service",
            variables=dict(
                connection_type="confluent-kafka-connect",
                key="tmp/mykey",
                dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
                new_streaming_system_name="test",
                new_streaming_system_type="confluent-cloud",
                new_cluster_id="test_cluster_id",
                new_cluster_name="test_cluster_name",
                new_cluster_type="confluent-kafka-connect",
            ),
        )
        self._validation_service_mock.test_new_credentials.assert_not_called()

    def test_onboard_streaming_cluster_connection_without_key(self):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data=Box(self._EXPECTED_NEW_CONNECTION_RESPONSE), errors=[]
        )
        self._validation_service_mock.test_new_credentials.return_value = "tmp/mykey"

        self._service.onboard_streaming_cluster_connection(
            connection_type="confluent-kafka-connect",
            api_key="api_key",
            secret="secret",
            url="http://confluent.com",
            dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
            new_streaming_system_name="test",
            new_streaming_system_type="confluent-cloud",
            new_cluster_id="test_cluster_id",
            new_cluster_name="test_cluster_name",
        )

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
            service="streaming_onboarding_service",
            variables=dict(
                connection_type="confluent-kafka-connect",
                key="tmp/mykey",
                dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
                new_streaming_system_name="test",
                new_streaming_system_type="confluent-cloud",
                new_cluster_id="test_cluster_id",
                new_cluster_name="test_cluster_name",
                new_cluster_type="confluent-kafka-connect",
                # following 3 only carried but not used in the call at all.
                api_key="api_key",
                secret="secret",
                url="http://confluent.com",
            ),
        )
        self._validation_service_mock.test_new_credentials.assert_called_once_with(
            connection_type="confluent-kafka-connect",
            dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
            cluster="test_cluster_id",
            api_key="api_key",
            secret="secret",
            confluent_env=None,
            url="http://confluent.com",
        )



from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.data_lake.spark import (
    SPARK_BINARY_MODE_CONFIG_TYPE,
    SPARK_DATABRICKS_CONFIG_TYPE,
    SPARK_HTTP_MODE_CONFIG_TYPE,
    SparkOnboardingService,
)
from montecarlodata.queries.onboarding import (
    TEST_SPARK_BINARY_MODE_CRED_MUTATION,
    TEST_SPARK_DATABRICKS_CRED_MUTATION,
    TEST_SPARK_HTTP_MODE_CRED_MUTATION,
)
from montecarlodata.utils import AwsClientWrapper, GqlWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class SparkOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = SparkOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    @patch.object(SparkOnboardingService, "onboard")
    def test_spark_binary_mode_flow(self, onboard_mock):
        options = {
            "host": "host",
            "database": "database",
            "port": 10000,
            "username": "user",
            "password": "password",
        }

        self._service.onboard_spark(
            SPARK_BINARY_MODE_CONFIG_TYPE, **options, **_SAMPLE_BASE_OPTIONS
        )
        self._assert_onboard(TEST_SPARK_BINARY_MODE_CRED_MUTATION, options, onboard_mock)

    @patch.object(SparkOnboardingService, "onboard")
    def test_spark_http_mode_flow(self, onboard_mock):
        options = {"url": "url", "username": "user", "password": "password"}

        self._service.onboard_spark(SPARK_HTTP_MODE_CONFIG_TYPE, **options, **_SAMPLE_BASE_OPTIONS)
        self._assert_onboard(TEST_SPARK_HTTP_MODE_CRED_MUTATION, options, onboard_mock)

    @patch.object(SparkOnboardingService, "onboard")
    def test_spark_databricks_flow(self, onboard_mock):
        options = {
            "workspace_url": "workspace_url",
            "workspace_id": "workspace_id",
            "cluster_id": "cluster_id",
            "token": "token",
        }

        self._service.onboard_spark(SPARK_DATABRICKS_CONFIG_TYPE, **options, **_SAMPLE_BASE_OPTIONS)
        self._assert_onboard(TEST_SPARK_DATABRICKS_CRED_MUTATION, options, onboard_mock)

    @staticmethod
    def _assert_onboard(mutation, options, onboard_mock):
        expected_options = {
            **options,
            **_SAMPLE_BASE_OPTIONS,
            **{"connectionType": "spark"},
        }

        onboard_mock.assert_called_once_with(
            validation_query=mutation,
            validation_response="testSparkCredentials",
            connection_type="spark",
            **expected_options,
        )



from unittest import TestCase
from unittest.mock import Mock

from montecarlodata.common.user import UserService
from montecarlodata.integrations.info.status import OnboardingStatusService
from tests.helpers import capture_function
from tests.test_common_user import _SAMPLE_CONFIG


class OnboardingStatusTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)

        self._service = OnboardingStatusService(
            _SAMPLE_CONFIG,
            command_name="test",
            user_service=self._user_service_mock,
        )

    def test_display_integrations_with_no_connections(self):
        self._user_service_mock.warehouses = None
        self._user_service_mock.bi_containers = None
        self._user_service_mock.etl_containers = None

        std_out = capture_function(
            self._service.display_integrations, {"table_format": "plain"}
        ).std_out
        self.assertEqual(
            std_out.getvalue().strip(),
            "Integration    Name    ID    Connection    Created on (UTC)",
        )

    def test_display_integrations_with_connections(self):
        (
            sample_uuid,
            sample_type1,
            sample_type2,
            sample_time,
            sample_uuid3,
            sample_type3,
        ) = ("1", "foo", "bar", "now", "3", "fivetran")
        sample_connection_identifier1 = {"key": "host", "value": "foo"}
        sample_connection_identifier2 = {"key": "host", "value": "bar"}
        sample_connection_identifier3 = {"key": "host", "value": "bar3"}
        self._user_service_mock.warehouses = [
            {
                "name": "Sample 1",
                "connections": [
                    {
                        "uuid": sample_uuid,
                        "type": sample_type1,
                        "connectionIdentifiers": [
                            sample_connection_identifier1,
                            sample_connection_identifier2,
                        ],
                        "createdOn": sample_time,
                    }
                ],
            }
        ]
        self._user_service_mock.bi_containers = [
            {
                "name": "Sample 2",
                "connections": [
                    {
                        "uuid": sample_uuid,
                        "type": sample_type2,
                        "connectionIdentifiers": [sample_connection_identifier2],
                        "createdOn": sample_time,
                    }
                ],
            }
        ]
        self._user_service_mock.etl_containers = [
            {
                "name": "Test Fivetran",
                "connections": [
                    {
                        "uuid": sample_uuid3,
                        "type": sample_type3,
                        "connectionIdentifiers": [sample_connection_identifier3],
                        "createdOn": sample_time,
                    }
                ],
            }
        ]

        expected_output = (
            "Integration    Name             ID  Connection    Created on (UTC)\n"
            "foo            Sample 1          1  host: foo     now\n"
            "                                    host: bar\n"
            "bar            Sample 2          1  host: bar     now\n"
            "Fivetran       Test Fivetran     3  host: bar3    now"
        )
        std_out = capture_function(
            self._service.display_integrations, {"table_format": "plain"}
        ).std_out
        self.assertEqual(std_out.getvalue().strip(), expected_output)



import configparser
import os
from dataclasses import dataclass
from typing import Optional

import click

import montecarlodata.settings as settings
from montecarlodata.fs_utils import mkdirs


@dataclass
class Config:
    mcd_id: str
    mcd_token: str
    mcd_api_endpoint: str
    mcd_agent_image_host: Optional[str] = None
    mcd_agent_image_org: Optional[str] = None
    mcd_agent_image_repo: Optional[str] = None


class ConfigManager:
    """
    Token is generated at https://github.com/monte-carlo-data/monolith-django/blob/a18ffc64ea2517166e66f0d3e85417b582398513/monolith/service/account.py#L236
    with the function call as `secrets.token_urlsafe(42)`, which is always 56 characters
    Details in https://docs.python.org/3/library/secrets.html#secrets.token_urlsafe
    """

    TOKEN_LENGTH = 56
    MCD_TOKEN = "mcd_token"

    def __init__(
        self,
        profile_name: str,
        base_path: str,
        config_parser: Optional[configparser.ConfigParser] = None,
    ):
        self._profile_name = profile_name
        self._base_path = base_path
        self._profile_config_file = os.path.join(self._base_path, settings.PROFILE_FILE_NAME)

        self._config = config_parser or configparser.ConfigParser()
        self._config.read(self._profile_config_file)

    """
        There is a circular dependency between ConfigManager and montecarlodata.errors
        so the decorator is imported here instead at top of the file with the other local imports.
        TODO: Move the config class into common.
    """
    from montecarlodata.errors import manage_errors

    @manage_errors
    def write(self, **kwargs) -> None:
        """
        Write any configuration key value pairs to the specified section (profile name)
        """
        if self._profile_name not in self._config.sections():
            # if the section does not exist add it
            self._config.add_section(self._profile_name)

        for k, v in kwargs.items():
            if k == self.MCD_TOKEN and len(v) != self.TOKEN_LENGTH:
                raise ValueError(
                    f"{self.MCD_TOKEN} received should have {self.TOKEN_LENGTH} length "
                    f"but received {len(v)}"
                )
            self._config.set(self._profile_name, k, v)

        mkdirs(self._base_path)
        with open(self._profile_config_file, "w") as cf:
            self._config.write(cf)

    def read(self) -> Optional[Config]:
        """
        Return configuration from section (profile name) if it exists.
        Any MCD values can be overwritten by the environment. Uses system default for AWS
        if not set.
        """
        try:
            return Config(
                mcd_id=settings.MCD_DEFAULT_API_ID
                or self._config.get(self._profile_name, "mcd_id"),
                mcd_token=settings.MCD_DEFAULT_API_TOKEN
                or self._config.get(self._profile_name, "mcd_token"),
                mcd_api_endpoint=settings.MCD_API_ENDPOINT
                or self._config.get(
                    self._profile_name,
                    "mcd_api_endpoint",
                    fallback=settings.MCD_DEFAULT_API_ENDPOINT,
                ),
                mcd_agent_image_host=settings.MCD_AGENT_IMAGE_HOST
                or self._config.get(
                    self._profile_name,
                    "mcd_agent_image_host",
                    fallback=settings.MCD_DEFAULT_AGENT_IMAGE_HOST,
                ),
                mcd_agent_image_org=settings.MCD_AGENT_IMAGE_ORG
                or self._config.get(
                    self._profile_name,
                    "mcd_agent_image_org",
                    fallback=settings.MCD_DEFAULT_AGENT_IMAGE_ORG,
                ),
                mcd_agent_image_repo=settings.MCD_AGENT_IMAGE_REPO
                or self._config.get(
                    self._profile_name,
                    "mcd_agent_image_repo",
                    fallback=settings.MCD_DEFAULT_AGENT_IMAGE_REPO,
                ),
            )
        except configparser.NoSectionError:
            click.echo(
                f"Failed to find configuration for '{self._profile_name}'. "
                "Please setup using 'configure' first"
            )



from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.data import ValidationResult
from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.etl.fivetran import (
    FivetranOnboardingService,
)
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_ADD_ETL_CONNECTION_RESPONSE_FIELD,
    EXPECTED_FIVETRAN_RESPONSE_FIELD,
)
from montecarlodata.queries.onboarding import (
    ADD_ETL_CONNECTION_MUTATION,
    TEST_FIVETRAN_CRED_MUTATION,
)
from montecarlodata.utils import AwsClientWrapper, GqlWrapper
from tests.test_common_user import _SAMPLE_CONFIG

_ETL_CONFIG = {
    "uuid": "my-snowflake-warehouse-uuid",
    "name": "fivetran-test-connection",
    "connectionType": "fivetran",
    "fivetranApiKey": "my-fivetran-api-key",
    "fivetranApiPassword": "my-fivetran-api-password",
    "fivetranBaseUrl": "https://api.fivetran.com",
    "collectorId": "my-collector-id",
}


class FivetranOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = FivetranOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )
        self._service._disable_handle_errors = True

    @patch.object(FivetranOnboardingService, "_validate_connection")
    @patch("montecarlodata.integrations.onboarding.base.prompt_connection")
    def test_onboard_fivetran(self, prompt_mock, validation_mock):
        self._user_service_mock.warehouses = [_ETL_CONFIG]
        validation_mock.return_value = ValidationResult(has_warnings=False, credentials_key="path")

        self._service.onboard_fivetran(etl_name=_ETL_CONFIG["name"])

        validation_mock.assert_called_once_with(
            query=TEST_FIVETRAN_CRED_MUTATION,
            response_field=EXPECTED_FIVETRAN_RESPONSE_FIELD,
        )

        self._request_wrapper_mock.make_request_v2.assert_called_with(
            query=ADD_ETL_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_ETL_CONNECTION_RESPONSE_FIELD,
            service="onboarding_service",
            variables={
                "key": "path",
                "name": _ETL_CONFIG["name"],
                "connectionType": _ETL_CONFIG["connectionType"],
            },
        )
